import { makeTick, } from './makeTick';
import { processAndDisposeOfTensor, wrapGenerator, } from './utils';
export const isWarmupSizeByPatchSize = (size) => {
    if (!size || typeof size !== 'object') {
        return false;
    }
    return 'patchSize' in size && typeof size.patchSize === 'number';
};
export const isNumericWarmupSize = (size) => {
    return Boolean(size) && typeof size === 'number' && size > 0;
};
const ERROR_INVALID_WARMUP_VALUE_URL = 'https://upscalerjs.com/documentation/troubleshooting#invalid-warmup-value';
export const ERROR_INVALID_WARMUP_VALUE = (size) => ([
    'Invalid value passed to warmup in warmupSizes:',
    JSON.stringify(size),
    `For more information, see ${ERROR_INVALID_WARMUP_VALUE_URL}.`,
].join('\n'));
export const getInvalidValueError = (size) => new Error(ERROR_INVALID_WARMUP_VALUE(size));
const getWidthAndHeight = (size) => {
    if (isWarmupSizeByPatchSize(size)) {
        const { patchSize, } = size;
        return patchSize;
    }
    return size;
};
export async function* warmup(tf, modelPackage, sizes) {
    const { model, modelDefinition, } = await modelPackage;
    for (const size of sizes) {
        if (!isWarmupSizeByPatchSize(size) && !isNumericWarmupSize(size)) {
            throw getInvalidValueError(size);
        }
        const warmupSize = getWidthAndHeight(size);
        let dummyTensor = tf.zeros([1, warmupSize, warmupSize, 3,]);
        yield [dummyTensor,];
        const fns = [
            modelDefinition.preprocess,
            (t) => model.predict(t),
            modelDefinition.postprocess,
        ].filter(Boolean);
        for (const fn of fns) {
            dummyTensor = processAndDisposeOfTensor(tf, dummyTensor, fn);
            yield [dummyTensor,];
        }
        dummyTensor.dispose();
        yield;
    }
}
export const getSizesAsArray = (sizes) => {
    if (Array.isArray(sizes)) {
        for (const size of sizes) {
            if (!isWarmupSizeByPatchSize(size) && !isNumericWarmupSize(size)) {
                throw getInvalidValueError(sizes);
            }
        }
        return sizes;
    }
    else if (isWarmupSizeByPatchSize(sizes) || isNumericWarmupSize(sizes)) {
        return [sizes,];
    }
    throw getInvalidValueError(sizes);
};
export const cancellableWarmup = async (tf, modelPackage, sizes, { signal = undefined, awaitNextFrame = false, } = {}, internalArgs) => {
    const tick = makeTick(tf, signal || internalArgs.signal, awaitNextFrame);
    await tick();
    await wrapGenerator(warmup(tf, modelPackage, getSizesAsArray(sizes)), tick);
};
